from jsondler.json_tools import get_paths_order


def sort_pandas_df(df, json_col_name, prior_list, reverse=False, inplace=True):
    paths_order = get_paths_order(in_json=list(df[json_col_name]), prior_list=prior_list, reverse=reverse)
    if inplace:
        df = df.reindex([path[0] for path in paths_order]).reset_index(drop=True)
    else:
        return df.reindex([path[0] for path in paths_order]).reset_index(drop=True)



import os
import unittest
import json

from jsondler.constants import TESTS_PATH, jsondler_logger
from jsondler import json_tools


PACKAGE_TEST_DATA_DIR = "json_tools"
INPUT_DIR = os.path.join(TESTS_PATH, 'test_data', PACKAGE_TEST_DATA_DIR)
OUTPUT_DIR = os.path.join(TESTS_PATH, 'test_results', PACKAGE_TEST_DATA_DIR)

try:
    os.makedirs(OUTPUT_DIR)
except OSError:
    pass


class TestRequests(unittest.TestCase):

    def test_get_by_path(self):
        target_json = {
            "list1": [3, 4, 5],
            "list2": [{"C": 3, "D": 4, "E": 5}, {"F": 6, "G": 7}, {"C": 8, "F": 9}],
            "A": 1,
            "B": 2,
            "dict1": {"key1": "a", "key2": "b"},
            "dict2": {"l1": ["a", "b"], "l2": ["c", "d"]},
        }

        path = ("*", 0, ["C", "D"])
        exp_res = [
            (["list2", 0, "C"], 3),
            (["list2", 0, "D"], 4),
        ]
        res = json_tools.get_by_path(in_json=target_json, path_list=path)
        self.assertEqual(exp_res, res)

    def test_deepupdate(self):
        d1 = {"k1": 1, "k2": 2, "k3": {"k3_1": 31, "k3_2": 32, "k_3_3": [33, 34, 35]}}
        d2 = {"k1": 1, "k2": 22, "k3": {"k3_2": 32, "k_3_3": [33, 44, 35]}}
        exp_res = {"k1": 1, "k2": 22, "k3": {"k3_1": 31, "k3_2": 32, "k_3_3": [33, 44, 35]}}
        res = json_tools.deepupdate(d=d1, u=d2)
        self.assertEqual(exp_res, res)

    def test_deepdiff(self):
        d1 = {"k1": 1, "k2": 2, "k3": {"k3_1": 31, "k3_2": 32, "k_3_3": [33, 34, 35]}}
        d2 = {"k1": 1, "k2": 22, "k3": {"k3_2": 32, "k_3_3": [33, 44, 35]}}
        exp_res = {"k2": 22, "k3": {"k3_1": None, "k_3_3": [33, 44, 35]}}
        res = json_tools.deepdiff(d1=d1, d2=d2)
        self.assertEqual(exp_res, res)


if __name__ == "main":
    unittest.main()



from types import GeneratorType
from itertools import tee
from collections import OrderedDict

import numpy as np

from jsondler.json_tools import columnize_dicts_list


class ColsDict(OrderedDict):
    """

    """

    def __init__(self, seq=None, **kwargs):
        seq = ColsDict._prepare_seq(seq=seq)
        super(ColsDict, self).__init__(seq, **kwargs)
        self._columns = list(map(lambda it: it[0], seq))

    @staticmethod
    def _prepare_seq(seq):
        if type(seq) in (list, tuple, GeneratorType):
            types_set = set()
            seq, seq_ = tee(seq)
            for it in range(100):
                types_set.add(type(next(seq_)))
            types_list = list(types_set)
            if len(types_list) == 1 and types_list[0] is dict:
                seq = columnize_dicts_list(seq, na=np.nan).items()
        return seq

    @property
    def columns(self):
        keys = self.keys()
        c_i = 0
        while c_i < len(self._columns):
            if self._columns[c_i] not in keys:
                self._columns.pop(c_i)
            else:
                c_i += 1
        for key in keys:
            if key not in self._columns:
                self._columns.append(key)
        return self._columns

    def append_row(self, row):
        pass

    def insert_row(self, i, row):
        pass

    def pop_row(self, i):
        pass

    def rearrange_cols(self, ordered_cols):
        self._columns = ordered_cols

    def rename_cols(self, rename_dict):
        for old_col in rename_dict:
            self[rename_dict[old_col]] = self.pop(old_col)

    def values(self):
        cols_array = np.array([self[col] for col in self.columns])
        return np.transpose(cols_array)

    def fillna(self, na):
        pass

    @property
    def shape(self):
        return len(self[self.columns[0]]), len(self.columns)



from jsondler.json_tools.requests import get_by_path, deepupdate, deepdiff, query, where
from jsondler.json_tools.index import index_dicts_list
from jsondler.json_tools.restruct import columnize_dicts_list
from jsondler.json_tools.sort import sort_dicts_list, get_paths_order



from collections import defaultdict

import numpy as np


def columnize_dicts_list(dicts_list, na=np.nan):
    col_names = list()
    cols_dict = defaultdict(list)
    d_i = 0
    for dict_i in dicts_list:
        for col_name in col_names:
            cols_dict[col_name].append(dict_i.pop(col_name, np.nan))
        for key in dict_i:
            col_names.append(key)
            cols_dict[key] = [na]*d_i + [dict_i[key]]
        d_i += 1
    return cols_dict



import itertools

from jsondler.json_tools import get_by_path


def sort_dicts_list(in_json, prior_list, reverse=False):
    paths_order = get_paths_order(in_json=in_json, prior_list=prior_list, reverse=reverse)
    out_json = list()
    for path in paths_order:
        out_json.append(in_json[path[0]])
    [in_json.pop(path[0]) for path in sorted(paths_order, key=lambda p: p[0], reverse=True)]
    out_json.__iadd__(in_json)
    return out_json


def get_paths_order(in_json, prior_list, reverse=False, preorder=None):
    # returns list of paths (no regular expressions): [path_1, path_2, ..., path_n]
    paths_order = list()
    if preorder is None:
        curr_lev = sorted(get_by_path(in_json=in_json, path_list=prior_list[0]), key=lambda p: p[1], reverse=reverse)
        if len(prior_list) == 1:
            return list(map(lambda p: p[0], curr_lev))
        return get_paths_order(in_json=in_json,
                               prior_list=prior_list[1:],
                               reverse=reverse,
                               preorder=_group_by_value(curr_lev))

    for group in preorder:
        paths_to_get = _superpos_paths(real_paths=group, path_to_superpos=prior_list[0])
        curr_lev = sorted(itertools.chain.from_iterable(map(lambda path: get_by_path(in_json=in_json,
                                                                                     path_list=path),
                                                            paths_to_get)),
                          key=lambda p: p[1], reverse=reverse)
        if len(prior_list) == 1:
            paths_order.__iadd__(map(lambda p: tuple(p[0]), curr_lev))
        else:
            paths_order.__iadd__(get_paths_order(in_json=in_json,
                                                 prior_list=prior_list[1:],
                                                 reverse=reverse,
                                                 preorder=_group_by_value(curr_lev)))
    return paths_order


def _group_by_value(sorted_items):
    v = None
    groups_list = list()
    group_list = list()
    for item in sorted_items:
        if item[1] != v:
            if group_list:
                groups_list.append(group_list.copy())
            group_list = [item[0]]
            v = item[1]
        else:
            group_list.append(item[0])
    groups_list.append(group_list.copy())
    return groups_list


def _superpos_paths(real_paths, path_to_superpos):
    superposed_paths = [[[], True] for i in range(len(real_paths))]
    for i in range(len(path_to_superpos)):
        for j in range(len(real_paths)):
            if not superposed_paths[j][1]:
                continue
            if path_to_superpos[i] == "*":
                try:
                    superposed_paths[j][0].append(real_paths[j][i])
                except IndexError:
                    superposed_paths[j][1] = False
                    superposed_paths[j][0].__iadd__(path_to_superpos[i:])
            elif type(path_to_superpos[i]) in (list, tuple, set, frozenset):
                try:
                    if real_paths[j][i] in path_to_superpos[i]:
                        superposed_paths[j][0].append(real_paths[j][i])
                    else:
                        superposed_paths[j][1] = False
                        superposed_paths[j][0].__iadd__(path_to_superpos[i:])
                except IndexError:
                    superposed_paths[j][1] = False
                    superposed_paths[j][0].__iadd__(path_to_superpos[i:])
            else:
                try:
                    if real_paths[j][i] == path_to_superpos[i]:
                        superposed_paths[j][0].append(real_paths[j][i])
                    else:
                        superposed_paths[j][1] = False
                        superposed_paths[j][0].__iadd__(path_to_superpos[i:])
                except IndexError:
                    superposed_paths[j][1] = False
                    superposed_paths[j][0].__iadd__(path_to_superpos[i:])
    return set(map(lambda p: tuple(p[0]), superposed_paths))



from collections import OrderedDict

from jsondler.json_tools import get_by_path, sort_dicts_list, query, where


class DictsList(list):

    def __init__(self, seq=()):
        super(DictsList, self).__init__(seq)
        self._ix = OrderedDict()

    def sort(self, prior_list, reverse=False, inplace=True):
        # I have not found how to do it properly but I do not need standart sort method of the list object
        if inplace:
            i = 0
            for d in sort_dicts_list(in_json=list(self), prior_list=prior_list, reverse=reverse):
                try:
                    self[i] = d
                except IndexError:
                    print("WARNING: something went wrong: sorted list is bigger")
                    break
                i += 1
        else:
            return DictsList(sort_dicts_list(in_json=list(self), prior_list=prior_list, reverse=reverse))

    def get_by_path(self, path_list, prefix_list=None):
        return get_by_path(in_json=list(self), path_list=path_list, prefix_list=prefix_list)

    def query(self, condition_list, condition_tree="AND"):
        """

        :param condition_list: list of conditions with items (path_list, lambda x: True condition)
        :param condition_tree: conditions tree from condition_list that define True or False for got entries
        :return: DictsList of items for which condition tree is True
        """
        print("not implemented yet")
        return DictsList(self[path[0]] for path, value in query(in_json=list(self),
                                                                condition_list=condition_list,
                                                                condition_tree=condition_tree))

    def where(self, path_list, condition):
        return DictsList(self[path[0]] for path, value in where(in_json=list(self),
                                                                path_list=path_list,
                                                                condition=condition))

    def create_index(self, fileld_name):
        n = 0
        for elm in self:
            if elm[fileld_name] in self._ix.keys():
                self._ix[elm[fileld_name]].append(n)
            else:
                self._ix[elm[fileld_name]] = [n]
            n += 1

    @property
    def ix(self):
        # use DictsList.ix[key]
        ix_dict = OrderedDict()
        for key in self._ix:
            ix_dict[key] = DictsList([self[ix] for ix in self._ix[key]])
        return ix_dict


class LinkedDictsList(DictsList):
    pass



import operator
from collections import Mapping
from functools import reduce


def get_by_path(in_json, path_list, prefix_list=None):
    pairs_list = list()
    if type(in_json) not in (dict, list, tuple):
        return pairs_list
    if prefix_list is None:
        prefix_list = list()
    if len(path_list) == 1:
        if path_list[0] == "*":
            if type(in_json) is dict:
                for key in in_json:
                    pairs_list.append((prefix_list.__add__([key]), in_json[key]))
            else:
                for i in range(len(in_json)):
                    pairs_list.append((prefix_list.__add__([i]), in_json[i]))
        elif type(path_list[0]) in (list, tuple, set, frozenset):
            for path_part in path_list[0]:
                try:
                    pairs_list.append((prefix_list.__add__([path_part]), in_json[path_part]))
                except (KeyError, TypeError):
                    pass
        else:
            try:
                pairs_list = [(prefix_list.__add__([path_list[0]]), in_json[path_list[0]])]
            except (KeyError, IndexError, TypeError):
                pass
    elif len(path_list) > 1:
        if path_list[0] == "*":
            if type(in_json) is dict:
                for key in in_json:
                    pairs_list.__iadd__(get_by_path(in_json[key], path_list[1:], prefix_list.__add__([key])))
            else:
                for i in range(len(in_json)):
                    pairs_list.__iadd__(get_by_path(in_json[i], path_list[1:], prefix_list.__add__([i])))
        elif type(path_list[0]) in (list, tuple, set, frozenset):
            for path_part in path_list[0]:
                try:
                    pairs_list.__iadd__(get_by_path(in_json[path_part], path_list[1:],
                                                    prefix_list.__add__([path_part])))
                except (KeyError, TypeError):
                    pass
        else:
            try:
                pairs_list = get_by_path(in_json[path_list[0]], path_list[1:],
                                         prefix_list.__add__([path_list[0]]))
            except (KeyError, TypeError):
                pass
    return pairs_list


def deepupdate(d, u):
    if d is None:
        d = dict()
    if u is None:
        u = dict()
    for k, v in u.items():
        if isinstance(v, Mapping):
            d[k] = deepupdate(d.get(k, dict()), v)
        else:
            d[k] = v
    return d


def deepdiff(d1, d2):
    """
    Calculates the minimal dict that need to be the input as u to deepupdate to get d2 from d1
    :param d1: dict 1
    :param d2: dict 2
    :return: dict
    """
    diff_dict = dict()
    k1_set = set()

    for k1 in d1:
        if k1 in d2:
            if d1[k1] != d2[k1]:
                if type(d1[k1]) is dict and type(d2[k1]) is dict:
                    diff_dict_k1 = deepdiff(d1=d1[k1], d2=d2[k1])
                    if diff_dict_k1:
                        diff_dict[k1] = diff_dict_k1
                else:
                    diff_dict[k1] = d2[k1]
        else:
            diff_dict[k1] = None
        k1_set.add(k1)

    for k2 in d2:
        if k2 not in k1_set:
            diff_dict[k2] = d2[k2]

    return diff_dict


def tuplize_json_coord_pairs(in_dict, *json_paths):
    for json_path in json_paths:
        for pair in get_by_path(in_dict, json_path):
            if len(pair) == 2 and pair[1]:
                reduce(operator.getitem, pair[0][:-1], in_dict)[pair[0][-1]] = tuple(pair[1])
    return in_dict


def query(in_json, condition_list, condition_tree="AND"):
    """

    :param in_json: dict or list
    :param condition_list: list of conditions with items (path_list, lambda x: True condition)
    :param condition_tree: conditions tree from condition_list that define True or False for got entries
    :return: list of pairs (path_list, value) for which condition_tree is True
    """
    return


def where(in_json, path_list, condition):
    pairs_list = list()
    for json_path, value in get_by_path(in_json=in_json, path_list=path_list):
        if condition(value):
            pairs_list.append((json_path, value))
    return pairs_list



from functools import reduce
from operator import getitem
from abc import ABCMeta, abstractmethod


class JsonEntry(object, metaclass=ABCMeta):

    @classmethod
    @abstractmethod
    def attr_scheme(cls):
        """
        json scheme (the keys must match attribute names defined in __init__)
        CAUTION: if attribute is not defined in __init__ method (sublevel key in a json)
        don't include it to the result dict
        :return: {attr_name: (path, in, json, entry,), ...}
        """
        return dict()

    def get_json(self):
        res_json = dict()
        for attr in self.attr_scheme():
            keys = self.attr_scheme()[attr]
            keys_l = len(keys)
            for i in range(keys_l):
                if i > 0:
                    if i == keys_l - 1:
                        sub_res_json = reduce(getitem, keys[:i], res_json)
                        sub_res_json[keys[i]] = self.__dict__[attr]
                    else:
                        sub_res_json = reduce(getitem, keys[:i], res_json)
                        if keys[i] not in sub_res_json:
                            sub_res_json[keys[i]] = dict()
                else:
                    if i == keys_l - 1:
                        res_json[keys[i]] = self.__dict__[attr]
                    elif keys[i] not in res_json:
                        res_json[keys[i]] = dict()
            keys = None
        return res_json

    @classmethod
    def load_from_dict(cls, in_dict):
        json_entry = cls()
        for attr in cls.attr_scheme():
            try:
                json_entry.__dict__[attr] = reduce(getitem, cls.attr_scheme()[attr], in_dict)
            except (KeyError, TypeError):
                print("WARNING: the path '%s' is absent in the input dict - cannot find a value for '%s'" %
                      (str(cls.attr_scheme()[attr]), attr))
                continue
        return json_entry



import os
import unittest
import json

from jsondler.constants import TESTS_PATH, jsondler_logger
from jsondler import _lib


PACKAGE_TEST_DATA_DIR = "_lib"
INPUT_DIR = os.path.join(TESTS_PATH, 'test_data', PACKAGE_TEST_DATA_DIR)
OUTPUT_DIR = os.path.join(TESTS_PATH, 'test_results', PACKAGE_TEST_DATA_DIR)

try:
    os.makedirs(OUTPUT_DIR)
except OSError:
    pass


class TestGeneralUtils(unittest.TestCase):

    pass


class TestCoordsTools(unittest.TestCase):

    pass


if __name__ == "main":
    unittest.main()



import os
import unittest
import json

from jsondler.constants import TESTS_PATH, jsondler_logger
from jsondler import dicts_list


PACKAGE_TEST_DATA_DIR = "jsondler"
INPUT_DIR = os.path.join(TESTS_PATH, 'test_data', PACKAGE_TEST_DATA_DIR)
OUTPUT_DIR = os.path.join(TESTS_PATH, 'test_results', PACKAGE_TEST_DATA_DIR)

try:
    os.makedirs(OUTPUT_DIR)
except OSError:
    pass


class TestDictsList(unittest.TestCase):

    dicts_list_obj = dicts_list.DictsList([
        {"id": 1, "coords": {"x": 0, "y": 0}},
        {"id": 3, "coords": {"x": 2, "y": 0}},
        {"id": 2, "coords": {"x": 0, "y": 1}},
        {"id": 3, "coords": {"x": 1, "y": 0}},
        {"id": 10},
    ])

    def test_DictsList_sort(self):
        sorted_dicts_list = dicts_list.DictsList([
            {"id": 1, "coords": {"x": 0, "y": 0}},
            {"id": 2, "coords": {"x": 0, "y": 1}},
            {"id": 3, "coords": {"x": 1, "y": 0}},
            {"id": 3, "coords": {"x": 2, "y": 0}},
            {"id": 10},
        ])

        self.dicts_list_obj.sort(prior_list=(("*", "id"), ("*", "coords", "x")))
        self.assertEqual(sorted_dicts_list, self.dicts_list_obj)


if __name__ == "main":
    unittest.main()



from collections import OrderedDict

from jsondler.json_tools.requests import get_by_path


def index_dicts_list(in_dicts_list, by, path=False):
    indexed_data = OrderedDict()
    if not path:
        for elm in in_dicts_list:
            if elm[by] in indexed_data.keys():
                indexed_data[elm[by]].append(elm)
            else:
                indexed_data[elm[by]] = [elm]
    else:
        got_by_path = get_by_path(in_json=in_dicts_list, path_list=by)
        for got_pair in got_by_path:
            if got_pair[1] in indexed_data.keys():
                indexed_data[got_pair[1]].append(in_dicts_list[got_pair[0][0]])
            else:
                indexed_data[got_pair[1]] = [in_dicts_list[got_pair[0][0]]]
    return indexed_data



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='jsondler',
    version='0.0.2',
    author="Denis Moshensky",
    author_email="loven7doo@gmail.com",  
    description="JSON handler for Python",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/loven-doo/jsondler",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
    ],
    packages=find_packages(),
    package_data={},
    install_requires=[
        'numpy >= 1.15.1',
        'pandas >= 0.23.4',
    ],
    entry_points={
        'console_scripts': []
    }
)
